/*-----------------------------------------------------------------
/					 Fichier  CBombe.cpp						
/------------------------------------------------------------------
Role: dfinition de la classe CBombe
-------------------------------------------------------------------
Versions: (depuis regroupement du 22/12/02)
-------------------------------------------------------------------
23/12/02: regroupement des modifs de Fran et Guillaume
	Rajout des df. des fonctions  chargerImages(); supprimerImages();
	Modification du constructeur pr les besoins de l'anim.
	Modification de la fonction explose() qui permet d'afficher les bonnes
	images de flammes en fonction de la direction
	Modification de la fonction afficheToi()
	Augmentation du delai entre images car fonction dplacer rajoute lenteur
16/01/03 Francois - modifs pour les bombes telecommandees => maintenant une bombe
	cree avec une dureeDeVie nulle est une bombe telecommandee
	+ ajout du parametre "ecran" a CBombe::staticInit()
	+ simplification des routines de chargement d'images (staticInit)
------------------------------------------------------------------*/


#include "CBombe.h"
#include "CAvatar.h"

// Definition des variable Static
CRound * CBombe::_ptRound = NULL;
CBruitage CBombe::_tabBruitageExplosion[NB_BRUITAGES_EXPLOSION];
SDL_Surface** CBombe::_tabImages = NULL;
SDL_Surface** CBombe::_tabImagesBombeTelecommandee = NULL;


// TO_THINK:la fonction staticInit doit etre appele AVANT le constructeur
//////////////////////////////////////////////////////////////////
CBombe::CBombe(int dureeDeVie, int dureeDeVieFlamme, int portee,
				int xCase, int yCase, CAvatar * ptAvatarPoseur)
:CContenu(),
 
 _portee(portee),
 _dureeDeVieFlamme(dureeDeVieFlamme),
 _xCase(xCase),
 _yCase(yCase),
 _ptAvatarPoseur(ptAvatarPoseur),
 _imageCourante(0),
 _delai(DELAI_IMAGES_BOMBES)
{
	Uint32 tpsActuel = SDL_GetTicks();
	if (dureeDeVie == 0)
	{
		_image = _tabImagesBombeTelecommandee[0];
		_momentExplosion = 0;
	}
	else
	{
		_momentExplosion = tpsActuel + dureeDeVie;
		_image = _tabImages[0];
	}
	// Moment de l'explosion = moment actuel + duree de vie de la bombe
	
	_tpsProchaineImage = tpsActuel + DELAI_IMAGES_BOMBES;
	


	assert(verifierCoherence());
}


//////////////////////////////////////////////////////////////////
//Destructeur
//////////////////////////////////////////////////////////////////
CBombe::~CBombe()
{
	// Si mon poseur n'est pas mort depuis qu'il m'a pose
	// alors j'augmente le nombre de bombes qu'il peut poser
	if (_ptAvatarPoseur != NULL)
		_ptAvatarPoseur->incrementerNbBombesPosables();
}




////////////////////////////////////////////////////////////////////
//explose est appel quand le temps de duree de vie est ecoule
//ele pose une flamme sur la case de la bombe, essaie de la propager 
//dans les 4 directions, et analyse si la portee de la flamme est atteinte
////////////////////////////////////////////////////////////////////
void CBombe::explose()
{
	// Le son de l'explosion choisi au hasard parmi 5.
	int randomNB = (int)(NB_BRUITAGES_EXPLOSION * float(rand())/float(RAND_MAX));
	CBombe::_tabBruitageExplosion[randomNB].jouer();


	_ptAvatarPoseur->detacherBombe(this);
	int idistance = 1;
	int x = _xCase;
	int y = _yCase;
	bool continuerPropagation = true;

	CCase * caseActuelle = _ptRound->getCase(_xCase, _yCase);
	caseActuelle->poserFlamme(_dureeDeVieFlamme, CENTRAL, NORMAL);
        
    type_propagation typeFlamme = NORMAL;

//Propagation vers la droite.
		while ( idistance <= _portee && continuerPropagation)
		{
			x++;
			//si on peut se propager, on choisit l'image normale, sinon
			//celle de fin de flamme.
			if (! _ptAvatarPoseur->estInvincible() ) {
			

			if ( (x+1) < NB_CASES_L)
			{
				if ( (_ptRound->getCase(x+1, y)->peutSePropager()) && (idistance !=_portee) )
					typeFlamme = NORMAL;
				else
					typeFlamme = FIN_FLAMME;
			}
			else
				typeFlamme = FIN_FLAMME;
			}
			

			continuerPropagation = _ptRound->getCase(x, y)->poserFlamme(_dureeDeVieFlamme, DROITE, typeFlamme);
			idistance++;
		}


//Propagation vers la gauche
		x = _xCase;
		y = _yCase;
		idistance = 1;
		continuerPropagation = true;
		while ( idistance <= _portee && continuerPropagation)
		{
			x--;
			//si on peut se propager, on choisit l'image normale, sinon
			//celle de fin de flamme.
			if (! _ptAvatarPoseur->estInvincible() ) {
			if ( (x-1) > 0)
			{
				if ( (_ptRound->getCase(x-1, y)->peutSePropager()) && (idistance !=_portee))
					typeFlamme = NORMAL;
				else
					typeFlamme = FIN_FLAMME;
			}
			else
				typeFlamme = FIN_FLAMME;
			}

			continuerPropagation = _ptRound->getCase(x, y)->poserFlamme(_dureeDeVieFlamme, GAUCHE, typeFlamme);
			idistance++;
		}

//Propagation vers le bas
		x = _xCase;
		y = _yCase;
		idistance = 1;
		continuerPropagation = true;
		while ( idistance <= _portee && continuerPropagation)
		{
			y++;
			//si on peut se propager, on choisit l'image normale, sinon
			//celle de fin de flamme.
			
		if (! _ptAvatarPoseur->estInvincible() ) {
			if ( (y+1) < NB_CASES_H)
			{
				if ( (_ptRound->getCase(x, y+1)->peutSePropager()) && (idistance !=_portee))
					typeFlamme = NORMAL;
				else
					typeFlamme = FIN_FLAMME;
			}
			else
					typeFlamme = FIN_FLAMME;
			}	

			continuerPropagation = _ptRound->getCase(x, y)->poserFlamme(_dureeDeVieFlamme, BAS, typeFlamme);
			idistance++;
		}

		
//Propagation vers le haut
		x = _xCase;
		y = _yCase;
		idistance = 1;
		continuerPropagation = true;
		while ( idistance <= _portee && continuerPropagation)
		{
			y--;
			//si on peut se propager, on choisit l'image normale, sinon
			//celle de fin de flamme.
			if (! _ptAvatarPoseur->estInvincible() ) {
			if ( (y-1) > 0)
			{
				
				if ( (_ptRound->getCase(x, y-1)->peutSePropager()) && (idistance !=_portee))
					typeFlamme = NORMAL;
				else
					typeFlamme = FIN_FLAMME;
			}
			else
					typeFlamme = FIN_FLAMME;
			}
			
			continuerPropagation = _ptRound->getCase(x, y)->poserFlamme(_dureeDeVieFlamme, HAUT, typeFlamme);
			idistance++;
		}
		
	assert(verifierCoherence());
}



//////////////////////////////////////////////////////////////////
//on teste si la duree de vie est atteinte, si oui, la bombe explose
//////////////////////////////////////////////////////////////////
void CBombe::vis2()
{
	// seule une bombe non telecommandee peut exploser toute seule
	if (_momentExplosion != 0)
	{
		// Si l'heure H de l'explosion est atteinte, alors faire exploser la bombe
		if ( SDL_GetTicks() >= _momentExplosion)
		{
			_aDetruire = true;	// aDetruire doit etre positionne a 'true' avant
			explose();			// l'appel a explose (car sinon ca boucle)
		}
	}
	assert(verifierCoherence());
}


//////////////////////////////////////////////////////////////////
//Chargement des images et des sons communs a toutes les bombes 
//et donc statiques
//////////////////////////////////////////////////////////////////
void CBombe::staticInit(CRound * ptRound, SDL_Surface * screen)
{	
	CBombe::_ptRound = ptRound;
	int i;
// Chargement des sons des explosions
	for (i = 0; i<NB_BRUITAGES_EXPLOSION; i++)
	{
		string fichier = "explosion";
		char tmp2[255];
		char tmp;
		sprintf(tmp2, "%d", i);
		tmp = tmp2[0];
		fichier = fichier + tmp;
		fichier = fichier + ".wav";
		cout << "fichier:" << fichier << endl;
		CBombe::_tabBruitageExplosion[i].charger(fichier.c_str());
	}

//chargement des images pr l'animation d'une bombe normale
	CBombe::_tabImages = (SDL_Surface**)malloc(NB_IMAGES_BOMBES*sizeof(SDL_Surface*));
	assert (_tabImages != NULL);

	string tabNombres[NB_IMAGES_BOMBES] = {"0","1","2","3",
									"4", "5", "6", "7", "8", "9",
									"10", "11", "12", "13", "14",
									"15", "16", "17"};
	

	string sImgPath;
	for (i=0; i<NB_IMAGES_BOMBES; i++)
	{
		sImgPath = "images/bombe/bomb_";
		sImgPath.append(tabNombres[i]);
		sImgPath.append(".bmp");

		CBombe::_tabImages[i] = ImageLoad(sImgPath.c_str());
		SDL_SetColorKey(_tabImages[i], SDL_SRCCOLORKEY,SDL_MapRGB(screen->format, 0, 0, 255));
	}
	
	//chargement des images pr l'animation d'une bombe telecommandee
	CBombe::_tabImagesBombeTelecommandee = (SDL_Surface**)malloc(18*sizeof(SDL_Surface*));
	assert (_tabImagesBombeTelecommandee != NULL);
	for (i=0; i<NB_IMAGES_BOMBES_TELECOMMANDEES; i++)
	{
		sImgPath = "images/bombe/bomb_t";
		sImgPath.append(tabNombres[i]);
		sImgPath.append(".bmp");

		CBombe::_tabImagesBombeTelecommandee[i] = ImageLoad(sImgPath.c_str());
		SDL_SetColorKey(_tabImagesBombeTelecommandee[i], SDL_SRCCOLORKEY,SDL_MapRGB(screen->format, 0, 0, 255));
	}
	
	cout << "eee";
}




//////////////////////////////////////////////////////////////////
//liberation des images et des sons communs a toutes les bombes 
//et donc statiques
//////////////////////////////////////////////////////////////////
void CBombe::staticDestruct()
{
	int i;
	for (i = 0; i<NB_BRUITAGES_EXPLOSION; i++)
		CBombe::_tabBruitageExplosion[i].liberer();

	
	for (i = 0; i < NB_IMAGES_BOMBES; i++)
		SDL_FreeSurface(CBombe::_tabImages[i]);

	for (i = 0; i < NB_IMAGES_BOMBES_TELECOMMANDEES; i++)
		SDL_FreeSurface(CBombe::_tabImagesBombeTelecommandee[i]);

	delete[] _tabImages;
	delete[] _tabImagesBombeTelecommandee;
}


/*
//////////////////////////////////////////////////////////////////
//Ne sert a rien ici (destruction immdiate) mais a t pens pour
//permettre une animation d'agonie (!)
//////////////////////////////////////////////////////////////////
int CBombe::continueDestruction()
{
	_estBienDetruit = true;
	return -1;
}



//////////////////////////////////////////////////////////////////
// Renvoie false: un avatar ne peut pas passer sur une case ou se 
//trouve une bombe !
//////////////////////////////////////////////////////////////////
bool CBombe::estTraversable()
{
	return false;	
}



//////////////////////////////////////////////////////////////////
void CBombe::setAvatarPoseurNULL()
{
	_ptAvatarPoseur = NULL;
}

*/

//////////////////////////////////////////////////////////////////
//choix de l'image  afficher: si le temps depuis le dernier 
//changement d'image a depass le dlai, on change l'image.
//////////////////////////////////////////////////////////////////
void CBombe::afficheToi(SDL_Rect coord_info, SDL_Surface * screen)
{
	Uint32 tpsActuel = SDL_GetTicks();

	if (  tpsActuel > _tpsProchaineImage ) 
	{
		_imageCourante++;
		_tpsProchaineImage += _delai;
		
		if (_imageCourante >= NB_IMAGES_BOMBES_TELECOMMANDEES && _momentExplosion == 0)
			_imageCourante = 0;
		else if (_imageCourante >= NB_IMAGES_BOMBES )
			_imageCourante = NB_IMAGES_BOMBES - 1;

	}

	if (_momentExplosion == 0)
		_image = _tabImagesBombeTelecommandee[_imageCourante];
	else
		_image = _tabImages[_imageCourante];
	
 
	assert(screen != NULL);
	assert(_image != NULL);
	
	SDL_BlitSurface(_image, NULL, screen, &coord_info);

}


#ifdef DEBUG
bool CBombe::verifierCoherence()
{
	return ( _ptRound != NULL );
}
#endif

