/*-----------------------------------------------------------------
/					 Fichier  CCase.cpp						
/------------------------------------------------------------------
Role: Implmentation de la classe CCase
-------------------------------------------------------------------
Versions: (depuis regroupement du 22/12/02)
-------------------------------------------------------------------
23/12/02: regroupement des modifs de Fran et Guillaume
	Modif de la fonction poserFlamme qui transmet ses paramtres
	au constructeur de Cflamme.
	Implmentation de la fonction peutSePropager
8/01/03: (G) Creation d'un nouveau bonus PorteeMax quand destruction Mur
	Modif Frequence Bonus
12/01/03: creation de #define dans utils.h pour la probabilite des bonus
------------------------------------------------------------------*/

#include "CCase.h"

// Definition des variable Static
CRound *	CCase::_ptRound = NULL;
SDL_Surface* CCase::_imageMurDestructible = NULL;
SDL_Surface* CCase::_imageMurInDestructible = NULL;
SDL_Surface* CCase::_imageMurDestruction = NULL;


////////////////////////////////////////////////////////////
// Note:appeler la fonction staticInit AVANT le constructeur
////////////////////////////////////////////////////////////
CCase::CCase(type_case type, CContenu * ptContenuDeDepart)
:_momentDestruction(0)
{
	
	switch(type)
	{
		case MUR_DESTRUCTIBLE:
			_bMurDestructible = true; _bMurIndestructible = false;
			break;
		case MUR_INDESTRUCTIBLE:
			_bMurDestructible = false; _bMurIndestructible = true;
			break;
		case CASE_LIBRE:
			_bMurDestructible = false; _bMurIndestructible = false;
			break;
		default:
			assert(false);
			break;
	}

	if ( ptContenuDeDepart != NULL )
		push_back(ptContenuDeDepart);

	// Pour l'affichage: choix de l'image en fonction du type de mu
	if (_bMurDestructible)
		_image = _imageMurDestructible;

	else if (_bMurIndestructible)
		_image = _imageMurInDestructible;
	
	else _image = NULL;
}

////////////////////////////////////////////////////////////
CCase::~CCase()
{
}



////////////////////////////////////////////////////////////
///Chargement des images communes a toutes les instances
//////////////////////////////////////////////////////////////////////////
void CCase::staticInit(CRound * ptLeRound, string imageMurDestructible,
					   string imageMurInDestructible, string imageMurDestruction)
{
	CCase::_ptRound = ptLeRound;
	_imageMurDestructible = ImageLoad(imageMurDestructible.c_str());
	_imageMurInDestructible = ImageLoad(imageMurInDestructible.c_str());
	_imageMurDestruction = ImageLoad(imageMurDestruction.c_str());
	
}



////////////////////////////////////////////////////////////
///Dechargement des images communes a toutes les instances
//////////////////////////////////////////////////////////////////////////
void CCase::staticDestruct()
{
	SDL_FreeSurface(_imageMurDestructible);
	SDL_FreeSurface(_imageMurInDestructible);
	SDL_FreeSurface(_imageMurDestruction);
}

/*
//////////////////////////////////////////////////////////////////////////

bool CCase::estMurDestructible()
{
	assert(verifierCoherence());
    return _bMurDestructible;
}


//////////////////////////////////////////////////////////////////////////
bool CCase::estMurIndestructible()
{
	assert(verifierCoherence());
    return _bMurIndestructible;
}
*/



//////////////////////////////////////////////////////////////////////////
//Sert a passer dans l'etat case libre, quand un mur est detruit; peut liberer
//un bonus (calcule alatoirement
//Note:les valeurs seront eventuellement modifiee pr un meilleur equilibre de jeu
//////////////////////////////////////////////////////////////////////////
void CCase::passerEtatCaseLibre()
{
	float randomNB;
	
	//on passe dans l'etat "case libre"
	_bMurDestructible = false;
	_momentDestruction = 0;
	_image = NULL;

	// Creation d'un bonus au hasard (cf CRound)		
	randomNB = float(rand())/float(RAND_MAX);
	if ( randomNB < PROBA_BONUS_PORTEE_MAX)
		push_back(new CBonus(BONUS_PORTEE_MAX));

	else if ( randomNB < PROBA_BONUS_NB_BOMBES)
		push_back(new CBonus(BONUS_NB_BOMBES));

	else if (randomNB < PROBA_BONUS_PORTEE_BOMBE)
		push_back(new CBonus(BONUS_PORTEE_BOMBE));

	else if (randomNB < PROBA_BONUS_INVINCIBILITE)
		push_back(new CBonus(BONUS_INVINCIBILITE));

	else if (randomNB < PROBA_BONUS_VITESSE)
		push_back(new CBonus(BONUS_VITESSE));

	else if (randomNB < PROBA_BONUS_TELECOMMANDEE)
		push_back(new CBonus(BONUS_TELECOMMANDEE));

}



//////////////////////////////////////////////////////////////////////////
//Chaque case se gre toute seule: change d'etat (libre ou pas), et anime
//l'ensemble de ses contenus
//////////////////////////////////////////////////////////////////////////
void CCase::gereToi()
{
	assert(verifierCoherence());

	list<CContenu*>::iterator itContenu;
	list<CContenu*>::iterator itTmp;
	bool bAChangeDeCase;
	
	
	//si le moment de destruction est arrive, la case devient libre
	if (_momentDestruction != 0 && (SDL_GetTicks() >= _momentDestruction))
			passerEtatCaseLibre();

		
	else if (_bMurIndestructible | _bMurDestructible)
		return;
	else
	{
		// On parcourt le vector de contenus et pour chacun on
		// appelle la fonction vis(). Ensuite verifier s'il est
		// a detruire par un appel a adetruire(). Si oui, alors
		// le mettre dans le contenuer des "adetruire" et le virer 
		for (itContenu = this->begin(); itContenu != this->end(); itContenu++)
		{
			if ( ! (*itContenu)->aDejaVecu() && ! (*itContenu)->estADetruire() )
			{
				(*itContenu)-> vis();

				// Si le contenu considere a change de case lors de son tour de jeu,
				// alors il faut le supprimer des contenus references par cette case
				// (il s'est tout seul reference aupres de sa case d'arrivee)
				bAChangeDeCase = (*itContenu)->aChangeDeCase();
				if (bAChangeDeCase)
				{
  					(*itContenu)->setAChangeDeCaseFalse();
					itTmp = itContenu;
					itContenu--;
					erase(itTmp);
				}
			}
		}


		// On parcourt les Contenus pour supprimer ceux qui sont a detruire
		// (Rque: les 2 parcours sont differents car un une bombe se trouvant
		// a la fin du conteneur peut par son explosion enflammer un avatar
		// se trouvant au debut et ainsi le positionner comme etant "aDetruire"
		for (itContenu = this->begin(); itContenu != this->end(); itContenu++)
		{
			if ( (*itContenu)->estADetruire() )
			{
				_ptRound->ajouterContenuADetruire(*itContenu);
				itTmp = itContenu;
				itContenu--;
				this->erase(itTmp);
			}
		}
	}
}


//////////////////////////////////////////////////////////////////////////
//teste si la flamme peut se propager. Est appele par explose de la classe Bombe
//Est utilise uniquement pr voir si il faut mettre une image de fin de flamme 
//ou pas
//////////////////////////////////////////////////////////////////////////
bool CCase::peutSePropager()
{
	if (_bMurIndestructible)
		return false;
	else if (!_bMurDestructible)	// cas d'une case libre pour tous 
			return true;

	else // cas d'un mur destructible
		return true;
}


//////////////////////////////////////////////////////////////////////////
//pose une flamme si la case est libre
//renvoie un booleen indiquant si il y a propagation de flamme (false si arrete p
//par un mur 
//////////////////////////////////////////////////////////////////////////
bool CCase::poserFlamme(int dureeDeVieFlamme, sens_propagation sens, type_propagation type)
{
	
	assert(verifierCoherence());

    if (_bMurIndestructible)
		return false;
	else if (!_bMurDestructible)	// cas d'une case libre pour tous 
	{	// les elements du conteneur, appeler leur fonction enflammeToi()
		// puis creer une nouvelle flamme et l'inserer dans le conteneur
		list<CContenu*>::iterator itContenu = this->begin();
		for (;itContenu != this->end(); itContenu++)
			(*itContenu)->enflammeToi();

		push_back(new CFlamme(dureeDeVieFlamme, sens, type));
		return true;
	}
	
	// cas d'un mur destructible qui n'a pas encore ete touche par unne flamme
	// utile pr super saien mode
	else if (_momentDestruction == 0) 
	{
		_momentDestruction = SDL_GetTicks() + DUREE_DESTRUCTION;
		_image = _imageMurDestruction;
		return false;
	}

	// on pose une deuxieme flamme sur un mur en cours de destruction
	//->> passe tt de suite en etat libre et ne libere pas de bonus
	//utile poura vancer vite 
	else 
	{
		//on passe dans l'etat "case libre"
		passerEtatCaseLibre();
		push_back(new CFlamme(dureeDeVieFlamme, sens, type));	
		return false;
	}
}



//////////////////////////////////////////////////////////////////////////
// renvoie un booleen indiquant si la case est traversable;
//////////////////////////////////////////////////////////////////////////
bool CCase::estTraversable()
{
	bool bTraversable = true;
	list<CContenu*>::iterator itContenu;

	// un mur n'est pas traversable...
	if (_bMurDestructible||_bMurIndestructible)
		return false;

	// mais pour une "case libre", cela depend de son contenu
	else
	{	
		itContenu = this->begin();
		for ( ; itContenu != this->end() && bTraversable; itContenu++)
			if ( ! (*itContenu)->estTraversable() )
				bTraversable = false;

		return bTraversable;
	}
}

//////////////////////////////////////////////////////////////////////////
// Parcourt tous les contenus en remettant leur variable "aDejaVecu"
// a false. Ainsi, ils sont pris en compte dans la boucle de jeu suivante.
//////////////////////////////////////////////////////////////////////////
void CCase::reinitialiserVieDeContenu()
{
	list<CContenu*>::iterator itContenu = this->begin();
	for ( ; itContenu != this->end(); itContenu++)
	{
		(*itContenu)->setADejaVecuFalse();
		(*itContenu)->setAChangeDeCaseFalse();
	}
}



//////////////////////////////////////////////////////////////////////////
//Affichage de la case: appel de la fonction d'affichage de tous les contenus
//////////////////////////////////////////////////////////////////////////

void CCase::afficheToi(SDL_Rect coord_info, SDL_Surface * screen)
{
	// En parcourant a l'envers, une bombe apparait
	// SOUS l'avatar qui vient de la poser.car la bombe
	//est apres l'avatar dans la liste!
	list<CContenu*>::reverse_iterator itContenu;

	// On affiche d'abord l'image de la case
	SDL_BlitSurface(_image,NULL,screen,&coord_info);
	// puis eventuellement ses contenus
	
		for (itContenu = this->rbegin(); itContenu != this->rend(); itContenu++)
			(*itContenu)->afficheToi(coord_info, screen);
}



#ifdef DEBUG
bool CCase::verifierCoherence()
{
	return ( (_ptRound!= NULL) 
			&& (_bMurDestructible && _bMurIndestructible)!= true);
}
#endif




